use glib::Object;

use crate::backend::Client;

glib::wrapper! {
    pub struct SetupWindow(ObjectSubclass<imp::SetupWindow>)
        @extends adw::Dialog, gtk::Widget,
        @implements gtk::gio::ActionGroup, gtk::gio::ActionMap, gtk::Accessible, gtk::Buildable,
            gtk::ConstraintTarget, gtk::Native, gtk::Root, gtk::ShortcutManager;
}

impl SetupWindow {
    pub fn new(client: Client) -> Self {
        Object::builder::<Self>()
            .property("client", &client)
            .build()
    }
}

pub mod imp {
    use std::cell::RefCell;

    use adw::prelude::AdwDialogExt;
    use adw::subclass::dialog::AdwDialogImpl;
    use glib::subclass::object::ObjectImpl;
    use glib::subclass::types::ObjectSubclass;
    use glib::subclass::InitializingObject;
    use gtk::prelude::ObjectExt;
    use gtk::subclass::prelude::*;
    use gtk::subclass::widget::WidgetImpl;
    use gtk::CompositeTemplate;

    use crate::backend::Client;
    use crate::gui::import_dialog::ImportDialog;

    #[derive(CompositeTemplate, Default, glib::Properties)]
    #[properties(wrapper_type = super::SetupWindow)]
    #[template(resource = "/ui/setup_window.ui")]
    pub struct SetupWindow {
        #[property(get, set)]
        client: RefCell<Client>,
    }

    #[gtk::template_callbacks]
    impl SetupWindow {
        #[template_callback]
        fn handle_start(&self) {
            self.obj().close();
        }

        #[template_callback]
        fn handle_import(&self) {
            let obj = self.obj();
            let import_dialog = ImportDialog::new(obj.client());
            import_dialog.present(Some(&*obj));
        }
    }

    #[glib::object_subclass]
    impl ObjectSubclass for SetupWindow {
        const NAME: &'static str = "TFSetupWindow";
        type Type = super::SetupWindow;
        type ParentType = adw::Dialog;

        fn class_init(klass: &mut Self::Class) {
            Self::bind_template(klass);
            Self::bind_template_callbacks(klass);
        }

        fn instance_init(obj: &InitializingObject<Self>) {
            obj.init_template();
        }
    }

    #[glib::derived_properties]
    impl ObjectImpl for SetupWindow {}

    impl WidgetImpl for SetupWindow {}
    impl AdwDialogImpl for SetupWindow {}
}
